% Paper Results Reproduce
% Czajkowski et al.(2014): Learning and fatigue effect revisited. Land Economics. 
% Created based on: Models for DCE by Prof.Mikolaj Czajkowski
% Qi Jiang/AEDE The Ohio State University/2021-07-18
% Email: jiang.1885@buckeyemail.osu.edu

clear all
clc

global B_backup;

%% ****************************  loading data  ****************************
% first read the excel file into matlab as column vectors. Then store it as
% a mat data file. 

% CT_dummy = dummyvar(CT);
% save cgg2015.mat


EstimOpt.DataFile = ('cgg2015.mat'); % specify the name of the data file
DATA = load(EstimOpt.DataFile); % load data, store it in a structure DATA

%% ****************************  model specification  ****************************


DATA.Y = DATA.CHOICE; % exlpanained variable (choice indicator)

DATA.Xa = [DATA.SQ, DATA.NAT1, DATA.NAT2, DATA.TRA1, DATA.TRA2, DATA.INF1, DATA.INF2, DATA.FEE]; % attributes
EstimOpt.NamesA = {'SQ';'NAT1';'NAT2';'TRA1';'TRA2';'INF1'; 'INF2'; 'FEE'}; % Names of the attributes

DATA.Xs = DummyVar(DATA.CT,1);  
EstimOpt.NamesS = {'CT2';'CT3';'CT4';'CT5';'CT6';'CT7';'CT8';'CT9';'CT10';'CT11';'CT12';'CT13';'CT14';'CT15';'CT16';'CT17';'CT18';'CT19';'CT20';'CT21';'CT22';'CT23';'CT24';'CT25';'CT26'};

EstimOpt.ProjectName = 'LearningFatigue'; % name of the project used for naming xls results files


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y)) == 1; % modify to include only some observations, e.g. model for male respondents only

INPUT.Y = DATA.Y(DATA.filter); 
INPUT.Xa = DATA.Xa(DATA.filter,:);
INPUT.Xs = DATA.Xs(DATA.filter,:); % Explanatory variables of scale

%% ****************************  sample characteristics ****************************
EstimOpt.NCT = 26; % Number of choice tasks per person 
EstimOpt.NAlt = 4; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; 


%% **************************** estimation and optimization options ****************************
% EstimOpt.eps = 1.e-9; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1e3; % maximum number of iterations
% OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton'
% EstimOpt.NumGrad = 1; % 0
% OptimOpt.GradObj = 'on'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'off'
% EstimOpt.HessEstFix = 1; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian
% EstimOpt.ApproxHess = 1;
% EstimOpt.Draws = 1;

% Estimopt.RealMin = 1; % use in the case of numerical errors (possibly for a few iterations only)

%% ****************************   Heteroskedastic MNL     ****************************

%  -29675.4509
% Results.MNL.b0 = [1.70092844482778;0.676512504857391;0.994954760065073;1.21609261726698;1.62374528614673;0.549112934545476;0.887173386157446;-1.13383934535298;0.0698108993885465;0.0560377623024355;0.0694446376876340;0.240949243166454;0.0856578740078140;0.161989376616357;0.0402568101384604;0.109596210712430;0.187091687203866;0.168611947075376;0.207361706586266;0.0878021780673264;0.171811554979739;0.129164545442475;0.207830946405767;0.305289803527630;0.221596291181793;0.288979598431781;0.118528028733589;0.332364063828921;0.243066518545446;0.232416809029718;0.203884979213040;0.213125586781381;0.168700900031848];

OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'
 
Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************   Heteroskedastic MXL_d     ****************************
EstimOpt.FullCov = 0; % Specify if random parametrs are correlated or not (0 = default)
EstimOpt.Dist = zeros(size(INPUT.Xa,2),1); % choose Xa parameters' distributions (0 - normal, -1 - constant, 1 - lognormal, 2 - Spike, 3 - triangular, 4 - Weibull, 5 - sinh-arcsinh, 6 - Johnson Sb, 7 - Johnson Su); 
% EstimOpt.Scores = 1; % Predict individual-specific paramter / WTP scores using bayes rule
 
% -17831.9844
Results.MXL_d.b0 = [-1.90908238891537;0.890823564104449;1.31720386550691;1.25073121560301;1.84084801787411;0.612003369406497;0.904667018030497;-2.84032275136389;5.10708756980056;0.482577194300689;0.790320484134234;0.410090309172535;0.748447331950678;0.207597748296718;0.455583641640740;2.84980165998230;0.169881280458744;0.160485924051647;0.225005640240675;0.382687050919647;0.344187410141976;0.355783596082234;0.510339093670142;0.449202582968261;0.455992067199362;0.582511010806284;0.583658529395205;0.503128777965666;0.543177585774473;0.506599075658101;0.509210394338222;0.539858318669209;0.637001725063725;0.598360270243810;0.430290870493385;0.591890596277757;0.501167468391449;0.463969682010050;0.525434598428613;0.482319733286795;0.497418310742714];

OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'
 
Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************  Heteroskedastic MXL     ****************************
EstimOpt.FullCov = 1;

% -16800.4168
Results.MXL.b0 = [-1.41147582998852;0.963787770570051;1.27758527167670;1.13775811327463;1.67123401859537;0.633439907106233;0.877763729245743;-3.20584628660234;3.64206899072027;-0.333200780308798;-0.680256020686428;-0.500931352835868;-0.847556398935318;-0.0805604038965933;-0.246447412563934;-1.14424327880188;0.826536177652082;1.22759546516647;0.316716005586462;0.519346763616887;0.0339255235466739;0.173172271825726;-0.397089737454248;-0.101649473661988;0.580519784017688;0.618010973991452;0.345945012426978;0.324118170867792;-0.219529635673258;0.355872918682519;0.869197723421994;0.0149159614006915;0.205244328356553;-0.253747762660951;0.100628666443295;-0.351561417384657;-0.419810112381685;0.793825597511053;0.0764789612220171;0.137521311569973;0.404178810047534;0.264649834732199;0.499652032561527;2.38479980107139;0.194609079716594;0.241383018532493;0.297480564129059;0.439483111831580;0.396919053421906;0.464547266159086;0.562300964525890;0.502235019034607;0.524487589343498;0.658217916326176;0.640376802690348;0.568415378891083;0.603130907676573;0.581252906993546;0.643429834792654;0.615694459885483;0.659037968245671;0.630369818007289;0.525199858879074;0.668712321433901;0.572488002285109;0.506963157065682;0.653620882530254;0.604779328572065;0.613990421937638];

OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'
 
Results.MXL = MXL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MXL = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************   Heteroskedastic GMXL_d     ****************************
EstimOpt.FullCov = 0;

EstimOpt.HessEstFix = 1;

EstimOpt.Gamma0 = 0.5; % 0 and 1 impose GMXL type II and I, respectively. Value in between allows for gamma to be freely estimated.

% -17362.8946 (gamma = 0)
% Results.GMXL_d.b0 = [-2.15779284408935;1.18098234513378;1.68450747668379;1.54921040804890;2.30492454150370;0.766342519270694;1.14364731983904;-3.54959591144831;6.82195461718086;0.596158059828557;0.899475055852411;0.101356386529015;0.612557465532836;0.229020553121539;0.438576330957596;3.79064722270382;0.169881280458744;0.160485924051647;0.225005640240675;0.382687050919647;0.344187410141976;0.355783596082234;0.510339093670142;0.449202582968261;0.455992067199362;0.582511010806284;0.583658529395205;0.503128777965666;0.543177585774473;0.506599075658101;0.509210394338222;0.539858318669209;0.637001725063725;0.598360270243810;0.430290870493385;0.591890596277757;0.501167468391449;0.463969682010050;0.525434598428613;0.482319733286795;0.497418310742714;0.788408251608084;0];

% -17335.9879 (gamma = free)
Results.GMXL_d.b0 = [-1.70778121478470;1.09761547040162;1.57816854459105;1.45357053560528;2.16168324342782;0.721426726293094;1.06848140626261;-3.23806118143031;5.90136724962795;0.523424721779235;0.810528043300162;0.0362011090401853;0.589603318861671;0.234305373792001;0.433298252198228;3.35246593599529;0.169881280458744;0.160485924051647;0.225005640240675;0.382687050919647;0.344187410141976;0.355783596082234;0.510339093670142;0.449202582968261;0.455992067199362;0.582511010806284;0.583658529395205;0.503128777965666;0.543177585774473;0.506599075658101;0.509210394338222;0.539858318669209;0.637001725063725;0.598360270243810;0.430290870493385;0.591890596277757;0.501167468391449;0.463969682010050;0.525434598428613;0.482319733286795;0.497418310742714;0.800584283639717;-0.723761714494825];


OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'
 
Results.GMXL_d = GMXL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.GMXL_d = GMXL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************  Heteroskedastic GMXL     ****************************
EstimOpt.FullCov = 1;

% -16709.0249 (gamma = 0)
% Results.GMXL.b0 = [-1.77805139664924;1.13800056604538;1.50592748140175;1.35379410266514;2.00255576302558;0.745047409069830;1.04421014109959;-3.72313901517610;4.82133085807134;-0.418022925037099;-0.856542008274282;-0.652584140217102;-1.09368970949527;-0.117959192504682;-0.335269133727695;-1.42693738531350;0.944395623884492;1.31725919863301;0.226926852771099;0.355450772485323;0.0118970233747202;0.139375417071468;0.0394442304202574;-0.210115289094869;0.516730665726065;0.391394524267559;0.370573967346830;0.274723716134329;0.345739550899073;0.518793367707157;1.11672315199192;-0.0382786120515815;0.148942931985694;-0.149095830525031;0.00888847376125330;-0.361958185068384;-0.630825681740415;0.298913302845973;0.176412632071273;0.0600888591467904;0.0629941386691197;0.0515303405617391;1.54532620718829;2.33084982296091;0.194609079716594;0.241383018532493;0.297480564129059;0.439483111831580;0.396919053421906;0.464547266159086;0.562300964525890;0.502235019034607;0.524487589343498;0.658217916326176;0.640376802690348;0.568415378891083;0.603130907676573;0.581252906993546;0.643429834792654;0.615694459885483;0.659037968245671;0.630369818007289;0.525199858879074;0.668712321433901;0.572488002285109;0.506963157065682;0.653620882530254;0.604779328572065;0.613990421937638;0.615437824311627;0];

% -16699.0434 (gamma = free)
Results.GMXL.b0 = [-2.35593476860915;1.19955431047991;1.60191892614717;1.43954419052651;2.12399738057619;0.762686513515993;1.06191829055113;-3.72459642209556;4.97273949855995;-0.363699869958299;-0.789642767270096;-0.497236015168029;-0.889518460061816;-0.0712700424858070;-0.282867018495751;-1.54752173798567;0.945762255711294;1.31026188175901;0.219754330152543;0.377721081081523;-0.0154058133297738;0.0991528353067139;-0.0971154008844089;-0.212841926370451;0.522623337937506;0.452368319514468;0.313612825037242;0.204277103799592;0.376103015891317;0.429504781408418;1.04421243167094;-0.0577257842880057;0.139388230348966;-0.144330122572573;0.0379189848577315;-0.336663984738737;-0.613225267978304;0.466897763397700;0.265904853637911;0.196881272039116;0.201028880194565;0.0989543977521053;1.73852507970651;1.85770401820124;0.194609079716594;0.241383018532493;0.297480564129059;0.439483111831580;0.396919053421906;0.464547266159086;0.562300964525890;0.502235019034607;0.524487589343498;0.658217916326176;0.640376802690348;0.568415378891083;0.603130907676573;0.581252906993546;0.643429834792654;0.615694459885483;0.659037968245671;0.630369818007289;0.525199858879074;0.668712321433901;0.572488002285109;0.506963157065682;0.653620882530254;0.604779328572065;0.613990421937638;0.717088514164962;-1.88440549632170];


OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'
 
Results.GMXL = GMXL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.GMXL = GMXL(INPUT,Results,EstimOpt,OptimOpt);


% %% ****************************  Scale MNL (TBD)    ****************************
% EstimOpt.FullCov = 0;
% 
% EstimOpt.Gamma0 = 0;
% % Results.GMXL_d.b0 =[-0.26;1.0;1.4;1.2;1.9;0.62;0.9;-2.83;0;0;0;0;0;0;0;0;0.09;0.17;0.21;0.34;0.25;0.29;0.25;0.30;0.29;0.39;0.35;0.33;0.29;0.37;0.41;0.41;0.35;0.42;0.33;0.47;0.34;0.36;0.41;0.37;0.37;0]
% % EstimOpt.Dist = -ones(size(INPUT.Xa,2),1); % choose Xa parameters' distributions (0 - normal, -1 - constant, 1 - lognormal, 2 - Spike, 3 - triangular, 4 - Weibull, 5 - sinh-arcsinh, 6 - Johnson Sb, 7 - Johnson Su); 
% 
% Results.GMXL_d = GMXL(INPUT,Results,EstimOpt,OptimOpt);



